//------------------------------------------------------------------------------
//title
//  pXvO
//ver
//  1.0
//designer
//  igeta kenichiro
//------------------------------------------------------------------------------

#include <machine.h>
#include <_h_c_lib.h>
#include "iodefine.h"
#include "lcd.h"

#define MPU_CLK         20000000UL                  //}CRg[Hz]
#define MPU_TIME        (1000000000L/MPU_CLK)       //}CR쎞[ns]
#define TIMEERA_LOOP    4096                        //[0.1us]
#define TIME_BASE       1000000L                    //[ns]
#define FREQ_RESOLUTION 10                          //g\[Hz]
#define CONVERT_BASE    (MPU_CLK/FREQ_RESOLUTION/2) //pXϊ萔(Ō2Ŋ͔̂ɂ邽)

#define SW_RUN          IO.PDR1.BIT.B4              //RUNSW
#define SW_RANGEL       IO.PDR1.BIT.B6              //WL
#define SW_RANGEH       IO.PDR1.BIT.B7              //WH
#define SW_FV_MODE      IO.PDRB.BIT.B4              //F/V[h
#define SW_AB_MODE      IO.PDRB.BIT.B5              //Q[h
#define SW_AUTO_MODE    IO.PDRB.BIT.B5              //AUTO[h
#define SW_START        IO.PDRB.BIT.B6              //X^[g/Xgbv
#define SW_DIR          IO.PDRB.BIT.B7              //؂ւ

//------------------------------------------------------------------------------

void TimerW_AB(void);
void PulseLoop(void);
void initialize(void);
void puts(char *str);
void Sleep(unsigned long time);
char *IntToAscii(unsigned int value, char *str, int radix, int width);
int ToFrequency(unsigned int data, unsigned int max);
void FVMode(void);
void ABMode(void);
void PulseMode(void);
void PulseRunMode(void);
void IntToStr(int data, char *str);
void main(void);

//------------------------------------------------------------------------------

volatile unsigned long gSystemTime;
volatile unsigned char gPwmOut;

volatile int gSetFrequency;         //ݒg
volatile int gFrequency;            //g
volatile int gSetMinFrequency;      //ݒŏg
volatile int gSlowUpRatio;          //X[Abv_El
volatile int gSetTime;              //g̎
volatile int gMode;                 //0:pX 1:F/V
volatile int gRangeSW;              //W؂ւrv̏
volatile int gPosition;             //ʒu

const int gRangeTable[] ={      //We[u[Hz]
     10100/FREQ_RESOLUTION,
     20200/FREQ_RESOLUTION,
     40400/FREQ_RESOLUTION,
    101000/FREQ_RESOLUTION
};

//------------------------------------------------------------------------------
//  ^C}[`
//  TIMEERA_LOOP[0.1us]Ŋ荞ݔ
//------------------------------------------------------------------------------

__interrupt(vect=19)
void INT_TimerA(void)
{
    static int count;

    count+=TIMEERA_LOOP;
    if(count >= 10000){
        count-=10000;
        gSystemTime += TIME_BASE/1000000;

        gRangeSW = SW_RANGEL | SW_RANGEH << 1;

        if(gMode == 0){
            PulseLoop();
        }
    }
    IRR1.BIT.IRRTA = 0;
}

//------------------------------------------------------------------------------
//  ^C}[u
//------------------------------------------------------------------------------

__interrupt(vect=22)
void INT_TimerV(void)
{
    if(TV.TCSRV.BIT.CMFB){
        TV.TCSRV.BIT.CMFB = 0;
        TV.TCORB = gPwmOut;
    }
}

//------------------------------------------------------------------------------
//  ^C}[v
//------------------------------------------------------------------------------

__interrupt(vect=21)
void INT_TimerW(void)
{
    unsigned int width; //pX
    unsigned int frequency;
    static int oldCapTimeA;
    static int oldCapTimeB;
    static int flagA;
    static int flagB;
    int out=0;

    if(gMode == 3){
        TimerW_AB();
    }
    //pX
    if(TW.TSRW.BIT.IMFC){                           //pXoH
        TW.TSRW.BIT.IMFC = 0;
        if(flagA == 0){
            width = TW.GRC - oldCapTimeA;           //pXvZ
            frequency = MPU_CLK/width;              //pXȂƂxvZ
            gFrequency = frequency>>(3-gRangeSW);
            out = (gFrequency>>(gRangeSW+2));
        }
        oldCapTimeA = TW.GRC;
        TW.GRA = TW.GRC+0x8000;                     //pXI[o[t[ݒ
        flagA = 0;
    }

    //pX
    if(TW.TSRW.BIT.IMFD){                           //pXoH
        TW.TSRW.BIT.IMFD = 0;
        if(flagB == 0){
            width = TW.GRD - oldCapTimeB;           //pXvZ
            frequency = MPU_CLK/width;              //pXȂƂxvZ
            gFrequency = frequency>>(3-gRangeSW);
            out = (gFrequency>>(gRangeSW+2));
        }
        oldCapTimeB = TW.GRD;
        TW.GRA = TW.GRD+0x8000;                     //pXI[o[t[ݒ
        flagB = 0;
    }

    //pX\͈͂𒴂Ăꍇ̏
    if(TW.TSRW.BIT.IMFA){                           //pXI[o[t[H
        TW.TSRW.BIT.IMFA = 0;
        TW.GRA = TW.GRA+0x8000;
        gFrequency=0;
        out = 0;
        flagA = flagB = 1;                          //pXI[o[t[tOZbg
    }

    if(out > 250-1){
        out = 250-1;
    }else if(out == 0){
        out = 250;
    }

    gPwmOut = out-1;
}

//------------------------------------------------------------------------------
//  QpXJEg
//------------------------------------------------------------------------------

void TimerW_AB(void)
{
    if(TW.TSRW.BIT.IMFC){                           //pXoH
        if(IO.PDR8.BIT.B3 == 1){
            if(IO.PDR8.BIT.B4 == 1){
                gPosition++;
            }else{
                gPosition--;
            }
        }else{
            if(IO.PDR8.BIT.B4 == 0){
                gPosition++;
            }else{
                gPosition--;
            }
        }
        TW.TSRW.BIT.IMFC = 0;
    }

    if(TW.TSRW.BIT.IMFD){                           //pXoH
        if(IO.PDR8.BIT.B4 == 1){
            if(IO.PDR8.BIT.B3 == 0){
                gPosition++;
            }else{
                gPosition--;
            }
        }else{
            if(IO.PDR8.BIT.B3 == 1){
                gPosition++;
            }else{
                gPosition--;
            }
        }
        TW.TSRW.BIT.IMFD = 0;
    }

}

//------------------------------------------------------------------------------
//  pXo͐
//------------------------------------------------------------------------------

void PulseLoop(void)
{
    const int table[] ={4, 8, 16, 40};

    static int frequency;
    int minFrequency;
    int setFrequency;
    int slowUpRatio;
    int out;

    setFrequency = gSetFrequency;
    minFrequency = gSetMinFrequency;
    slowUpRatio  = gSlowUpRatio;

    if(slowUpRatio == 0){
        frequency = setFrequency;

    }else if(frequency < setFrequency){
        frequency+=slowUpRatio;
        if(frequency > setFrequency){
            frequency = setFrequency;
        }

    }else if(frequency > setFrequency){
        frequency-=slowUpRatio;
        if(frequency < setFrequency){
            frequency = setFrequency;
        }
    }

    if(frequency == setFrequency){
        if(gSetTime > 0){
            gSetTime--;
            if(gSetTime == 0){
                gSetFrequency=0;
            }
        }

    }

    if(frequency >= minFrequency && frequency > CONVERT_BASE/65535){
        TW.GRC = CONVERT_BASE/(unsigned)frequency;  //gpXϊ
        TW.TMRW.BIT.CTS = 1;
        out = (frequency/table[gRangeSW]);

    }else{
        if(TW.TMRW.BIT.CTS == 1){
            TW.TCRW.BIT.TOA = 0;
            TW.TCRW.BIT.TOB = 0;
            TW.TMRW.BIT.CTS = 0;
        }
        out = 0;
    }
    if(out > 250-1){
        out = 250-1;
    }else if(out == 0){
        out = 250;
    }
    gPwmOut = out-1;
//  TV.TCORB = out-1;
}

//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------

void initialize(void)
{
    IO.PDR1.BYTE = 0x2f;
    IO.PCR1      = 0x2f;

    IO.PDR2.BYTE = 0xfd;
    IO.PCR2      = 0xfd;

    IO.PDR5.BYTE = 0xff;
    IO.PCR5      = 0xff;

    IO.PDR7.BYTE = 0xff;
    IO.PCR7      = 0xff;

    IO.PDR8.BYTE = 0xe1;
    IO.PCR8      = 0xe7;

    //`^cݒ
    AD.ADCSR.BYTE = 0x13;
    AD.ADCSR.BIT.ADST = 1;

    //^C}[`ݒ
    TA.TMA.BYTE = 0x06;
    IENR1.BIT.IENTA = 1;

    //^C}[uݒ
    //PWMg=MPU_CLK/4/250
    TV.TCRV0.BYTE = 0x89;
    TV.TCRV1.BYTE = 0x00;
    TV.TCSRV.BYTE = 0x09;
    TV.TCORA      = 250-1;
    TV.TCORB      = 250-1;

    //^C}[vݒ
    TW.TMRW.BYTE  = 0x10;
    TW.TCRW.BYTE  = 0x80;
    TW.TIERW.BYTE = 0x00;
    TW.TIOR0.BYTE = 0x00;
    TW.TIOR1.BYTE = 0x40;
    TW.GRB = 0;

    set_imask_ccr(0);               //݋

    //VA|[gݒ
    SCI3.SCR3.BYTE = 0x00;
    SCI3.SMR.BYTE = 0x00;
    SCI3.BRR = 15;
    IO.PMR1.BIT.TXD = 1;
    Sleep(10);
    SCI3.SCR3.BIT.TE = 1;
    SCI3.SCR3.BIT.RE = 1;
    SCI3.SCR3.BIT.RIE = 1;  //M݋

    //LCD̏
    LcdWrite(LCD_MODE_COMMAND, LCD_AREA_12, LCD_COMMAND_RESET);
    Sleep(10);
    LcdWrite(LCD_MODE_COMMAND, LCD_AREA_12, LCD_COMMAND_VIEW_ON);

    //LCDʏ
    LcdClr(LCD_AREA_12);
}

//------------------------------------------------------------------------------
//  o
//------------------------------------------------------------------------------

void puts(char *str)
{
    while(*str != 0){
        while(SCI3.SSR.BIT.TDRE==0){
            sleep();
        }
        SCI3.TDR = *str++;
    }
}

//------------------------------------------------------------------------------
//  ԑ҂[ms]
//------------------------------------------------------------------------------

void Sleep(unsigned long time)
{
    unsigned long oldTime = gSystemTime;

    while(gSystemTime-oldTime <= time){
        sleep();    //邱Ƃ̂Ń}CR~
    }
}

//------------------------------------------------------------------------------
//  unsigned int當ɕϊ
//------------------------------------------------------------------------------

char *IntToAscii(unsigned int value, char *str, int radix, int width)
{
    int n;
    char *buf = &(str[width-1]);

    str[width]=0;
    for(n=0;n < width;){
        *buf=value%radix+'0';
        if(*buf > '9'){
            *buf += 'a'-'9'-1;
        }
        value=value/radix;
        buf--;
        n++;
        if(value == 0){
            break;
        }
    }
    for(;n < width;n++){
        *buf=' ';
        buf--;
    }
    return(str);
}

//------------------------------------------------------------------------------
//  {[̒lgɕϊ
//------------------------------------------------------------------------------

int ToFrequency(unsigned int data, unsigned int max)
{
    unsigned long out;

    out = max*data;

    return(out>>16);
}

//------------------------------------------------------------------------------
//  F/Vϊ[h
//------------------------------------------------------------------------------

void FVMode(void)
{
    char str[5+1];

    gMode = 1;

    //^C}[vݒ
    TW.TMRW.BYTE  = 0x80;
    TW.TCRW.BYTE  = 0x00;
    TW.TIERW.BYTE = 0x0d;
    TW.TIOR0.BYTE = 0x00;
    TW.TIOR1.BYTE = 0x45;
    TW.GRA = 0x8000;

    LcdClr(LCD_AREA_12);
    LcdMov(0, 0);
    LcdPuts("F/V MODE");

    while(1){
        if(SW_FV_MODE == 0 || SW_AB_MODE == 1){
            break;
        }
        TW.TCRW.BYTE = (3-gRangeSW) << 4;

        IntToAscii(gFrequency, str, 10, 5);
        LcdMov(0, 1);
        LcdPuts("FREQ ");
        LcdPuts(str);
        LcdPuts("Hz");

        Sleep(10);
    }

    //^C}[vݒ
    TW.TMRW.BYTE  = 0x10;
    TW.TCRW.BYTE  = 0x80;
    TW.TIERW.BYTE = 0x00;
    TW.TIOR0.BYTE = 0x00;
    TW.TIOR1.BYTE = 0x40;
    TW.GRB = 0;

    gMode = 0;
}

//------------------------------------------------------------------------------
//  Q[h
//------------------------------------------------------------------------------

#define AB_MODE_TIME 200

void ABMode(void)
{
    int count=0;
    int dir=0;
    char str[5+1];
    int pulse;
    int time;

    gPosition=0;

    gMode = 3;

    //^C}[vݒ
    TW.TMRW.BYTE  = 0x80;
    TW.TCRW.BYTE  = 0x00;
    TW.TIERW.BYTE = 0x0c;
//  TW.TIERW.BYTE = 0x04;
    TW.TIOR0.BYTE = 0x00;
    TW.TIOR1.BYTE = 0x77;   //GbW
    TW.GRA = 0x8000;

    TV.TCRV0.BIT.CMIEB = 0;

    LcdClr(LCD_AREA_12);
    LcdMov(0, 0);
    LcdPuts("AB MODE");

    pulse = (gRangeSW+1)*200;
    IntToAscii(pulse, str, 10, 5);

    LcdPuts(str);
    puts(str);
    puts("stp ");

    time = (AD.ADDRA>>8)+10;
//  IntToAscii(1000/(time*2), str, 10, 5);
    IntToAscii(time*2, str, 10, 5);

    LcdPuts(str);
    LcdPuts("ms");
    puts(str);
    puts("ms\r\n");

    while(1){
        if(SW_FV_MODE == 0 || SW_AB_MODE == 0){
            break;
        }

        IntToAscii(count, str, 10, 5);
        LcdMov(0, 1);
        LcdPuts("CNT ");
        LcdPuts(str);

        IntToAscii(gPosition/count, str, 10, 5);
        LcdMov(0, 3);
        LcdPuts("    ");
        LcdPuts(str);

        if(gPosition >= 0){
            IntToAscii(gPosition, str, 10, 5);
        }else{
            IntToAscii(-gPosition, &(str[1]), 10, 4);
            str[0]='-';
        }
        LcdMov(0, 2);
        LcdPuts("POS ");
        LcdPuts(str);

        if(count >= 0){
            puts(str);
            puts("\r\n");
        }
        if(count >= pulse){
            dir = 1;
        }
        if(dir == 0){
            IO.PDR8.BIT.B1 = 1;
            Sleep(time);
            IO.PDR8.BIT.B1 = 0;
            Sleep(time);
            count++;

        }else if(count >= 0){
            IO.PDR8.BIT.B2 = 1;
            Sleep(time);
            IO.PDR8.BIT.B2 = 0;
            Sleep(time);
            count--;
        }
    }

    //^C}[vݒ
    TW.TMRW.BYTE  = 0x10;
    TW.TCRW.BYTE  = 0x80;
    TW.TIERW.BYTE = 0x00;
    TW.TIOR0.BYTE = 0x00;
    TW.TIOR1.BYTE = 0x40;
    TW.GRB = 0;

    TV.TCRV0.BIT.CMIEB = 1;

    gMode = 0;
}

//------------------------------------------------------------------------------
//  pXo̓[h
//------------------------------------------------------------------------------

void PulseMode(void)
{
    int frequency;
    int range;
    char str[5+1];
    unsigned int tmp;

    LcdClr(LCD_AREA_12);
    LcdMov(0, 0);
    LcdPuts("PULSE MODE");

    while(1){
        if(SW_FV_MODE == 1 || SW_AUTO_MODE == 1){
            break;
        }

        if(SW_DIR == 0){
            TW.TIOR0.BYTE = 0x03;
        }else{
            TW.TIOR0.BYTE = 0x30;
        }

        if(SW_RUN == 0){            //RUN SWꂽH
            while(SW_RUN == 0){     //RUN SW܂ő҂
                Sleep(10);
            }
            if(SW_DIR == 0){
                TW.TCRW.BIT.TOA = 1;
                Sleep(10);
                TW.TCRW.BIT.TOA = 0;
            }else{
                TW.TCRW.BIT.TOB = 1;
                Sleep(10);
                TW.TCRW.BIT.TOB = 0;
            }
        }

        range = gRangeSW;
        frequency = ToFrequency(AD.ADDRA, gRangeTable[range]);
        if(SW_START == 1){
            gSetFrequency = frequency;

        }else{
            gSetFrequency = 0;
        }
        gSetMinFrequency = ToFrequency(AD.ADDRB, gRangeTable[range]);
        tmp = AD.ADDRC;
        if(tmp > 64500){
            gSlowUpRatio = 0;
        }else{
            gSlowUpRatio = ToFrequency(tmp, gRangeTable[range])/100+1;
        }
        gSetTime = 0;

        IntToAscii(frequency, str, 10, 5);
        LcdMov(0, 1);
        LcdPuts("VR1:PLS ");
        LcdPuts(str);
        LcdPuts("0pps");

        IntToAscii(gSetMinFrequency, str, 10, 5);
        LcdMov(0, 2);
        LcdPuts("VR2:MIN ");
        LcdPuts(str);
        LcdPuts("0pps");

        IntToAscii(gSlowUpRatio, str, 10, 4);
        LcdMov(0, 3);
        LcdPuts("VR3:RAMP ");
        LcdPuts(str);
        LcdPuts("0pps/ms");

        Sleep(10);
    }

    gSetFrequency = 0;
}

//------------------------------------------------------------------------------
//  pXo̓[h(RUN[h)
//------------------------------------------------------------------------------

void PulseRunMode(void)
{
    int frequency;
    int minFrequency;
    int slowUpRatio;
    int time;
    unsigned int tmp;

    char str[5+1];
    static int range;

    LcdClr(LCD_AREA_12);
    LcdMov(0, 0);
    LcdPuts("PULSE RUN ");
    Sleep(500);

    while(1){
        if(SW_FV_MODE == 1 || SW_AUTO_MODE == 0){
            break;
        }

        if(SW_DIR == 0){
            TW.TIOR0.BYTE = 0x03;
        }else{
            TW.TIOR0.BYTE = 0x30;
        }

        range = gRangeSW;
        frequency = ToFrequency(AD.ADDRA, gRangeTable[range]);
        minFrequency = ToFrequency(AD.ADDRB, gRangeTable[range]);
        tmp = AD.ADDRC;
        if(tmp > 64500){
            slowUpRatio = 0;
        }else{
            slowUpRatio = ToFrequency(tmp, gRangeTable[range])/100+1;
        }
        time = AD.ADDRD/4+100;

        IntToAscii(frequency, str, 10, 5);
        LcdMov(0, 0);
        LcdPuts("VR1:PLS  ");
        LcdPuts(str);
        LcdPuts("0pps");

        IntToAscii(minFrequency, str, 10, 5);
        LcdMov(0, 1);
        LcdPuts("VR2:MIN  ");
        LcdPuts(str);
        LcdPuts("0pps");

        IntToAscii(frequency/slowUpRatio, str, 10, 5);
        LcdMov(0, 2);
        LcdPuts("VR3:RAMP  ");
        LcdPuts(str);
        LcdPuts("ms");

        IntToAscii(time, str, 10, 5);
        LcdMov(0, 3);
        LcdPuts("VR4:TIME  ");
        LcdPuts(str);
        LcdPuts("ms");

        Sleep(10);

        if(SW_RUN == 1){
            continue;
        }
        while(SW_RUN == 0){         //RUN SW܂ő҂
            Sleep(10);
        }
        gSetFrequency = frequency;
        gSetMinFrequency = minFrequency;
        gSlowUpRatio = slowUpRatio;
        gSetTime = time;
    }

    gSetTime = 0;
    gSetFrequency = 0;
}

//------------------------------------------------------------------------------
//  C
//------------------------------------------------------------------------------

void main(void)
{
    int sw;
    int mode=0;

    initialize();

    LcdMov(0, 0);
    LcdPuts("PULSE OSC VER 1.1");
    LcdMov(0, 1);   LcdPuts("SW1   SW2  SW3   SW4");
    LcdMov(0, 2);   LcdPuts("F/V   AUTO START CW");
    LcdMov(0, 3);   LcdPuts("PULSE MANU STOP  CCW");
    Sleep(2000);

    while(1){
        if(SW_FV_MODE == 1){                //F/V[hH
            if(SW_AB_MODE == 1){
                ABMode();
            }else{
                FVMode();
            }
        }else if(SW_AUTO_MODE == 1){            //pXo̓[h(RUN[h)H
            PulseRunMode();

        }else{
            PulseMode();
        }

        Sleep(1);
    }
}

//------------------------------------------------------------------------------
